#ifndef EXCHANGEDATA_H
#define EXCHANGEDATA_H

#include <QVector>
#include <QMutex>
#include "hidcontrol.h"

#define STOP 0
#define STOPED 1
#define STOP_NO 2

#define PAUSE 0
#define PAUSE_NO 1


// ADC0 - 0b00000001
// ADC1 - 0b00000010
// ADC2 - 0b00000100
// ADC2 - 0b00001000
#define ADC0_EN    0
#define ADC1_EN    1
#define ADC2_EN    2
#define ADC3_EN    3
// Быстрый режим ождин опрос ADC за проход (Если скорость более 250)
// FAST_REC - 0b1000xxxx
#define FAST_REC    7
// БЫстрый режим с уменьшенной частотой деления на ADC (Если скорость более 350)
// FAST_DIV - 0b0100xxxx
#define FAST_DIV    6


class ExchangeData : public QObject
{
Q_OBJECT

public slots:
    void run();
signals:
    void error(QString err);

public:
    ExchangeData();

    bool pause(bool);
    bool stop();

    QMutex mutex_stop;
    QMutex mutex_pause;
    QMutex mutex_usb;
    QMutex mutex_data_graph;
    QMutex mutex_value;


    bool connect();

    QString error_usb;

    int getFreqStart() ;
    void setFreqStart(int value);

    int getFreqStop() ;
    void setFreqStop(int value);

    int getFreqStep() ;
    void setFreqStep(int value);

    int getValueDelim1() ;
    void setValueDelim1(int value);

    int getValueDelim2() ;
    void setValueDelim2(int value);

    int getValueDelim3() ;
    void setValueDelim3(int value);

    int getValueDelim4() ;
    void setValueDelim4(int value);

    double getStep_dds() ;
    void setStep_dds(double value);

    int getSpeed_diff() ;
    void setSpeed_diff(int value);

    unsigned int getTimer_tick() ;
    void setTimer_tick(unsigned int value);

    QVector<double> *getX() ;
    void setX(int index,int value);

    QVector<double> *getY1() ;
    void setY1(int index,int value);

    QVector<double> *getY2() ;
    void setY2(int index,int value);

    QVector<double> *getY3() ;
    void setY3(int index,int value);

    QVector<double> *getY4() ;
    void setY4(int index,int value);

    hidControl *getHid() ;
    void setHid(hidControl *value);

    bool getDevice_connect() ;
    void setDevice_connect(bool value);

    int getErr_usb_count() ;
    void setErr_usb_count(int value);

    QString getError_usb();
    void setError_usb( QString &value);

    int getWork_pause() ;
    void setWork_pause(int value);

    int getWork_stop() ;
    void setWork_stop(int value);

    bool getChanelEnable1();
    void setChanelEnable1(bool value);

    bool getChanelEnable2();
    void setChanelEnable2(bool value);

    bool getChanelEnable3();
    void setChanelEnable3(bool value);

    bool getChanelEnable4();
    void setChanelEnable4(bool value);

    double getFreqGen();
    void setFreqGen(double value);

private:


    int work_pause;

    int work_stop;

    /// DDS
    int FreqStart;
    int FreqStop;
    int FreqStep;
    uint FreqGen;

    int ValueDelim1;
    int ValueDelim2;
    int ValueDelim3;
    int ValueDelim4;

    bool ChanelEnable1;
    bool ChanelEnable2;
    bool ChanelEnable3;
    bool ChanelEnable4;


    double step_dds;

    int speed_diff;

    unsigned int timer_tick;
    QVector<double> *x, *y1,*y2,*y3,*y4; // График
    bool graf_create; // График создан

    /// USB
    hidControl *hid;
    bool device_connect;
    int err_usb_count; // Количество ошибок
};

#endif // EXCHANGEDATA_H
