#ifndef HIDCONTROL_H
#define HIDCONTROL_H

#include <stdlib.h>
#include <unistd.h>

#include <QDir>
#include <QLibrary>
#include <QTextStream>


#include "hidapi.h"



#define REPORT_LENGTH   9

#define VID_NUMBER 5824
#define PID_NUMBER 1503



typedef struct hid_device_info* (*HID_ENUMERATE)(unsigned short, unsigned short);
typedef void (*HID_FREE_ENUMARATION)(struct hid_device_info*);
typedef hid_device* (*HID_OPEN)(unsigned short, unsigned short, wchar_t*);
typedef int (*HID_GET_MANUFACTURER_STRING)(hid_device *, wchar_t *, size_t);
typedef int (*HID_GET_PRODUCT_STRING)(hid_device*, wchar_t*, size_t);
typedef int (*HID_GET_SERIAL_NUMBER_STRING)(hid_device*, wchar_t*, size_t);
typedef int (*HID_SET_NONBLOCKING)(hid_device*, int);
typedef int (*HID_SEND_FEATURE_REPORT)(hid_device*, const unsigned char*, size_t);
typedef int (*HID_GET_FEATURE_REPORT)(hid_device*, unsigned char*, size_t);

struct dataexchange_t       // Описание структуры для передачи данных
{
    uint32_t data1;
    uint32_t data2;
}typedef dataexchange;


class hidControl
{
private:
    HID_OPEN hid_open;
    HID_SET_NONBLOCKING hid_set_nonblocking;
    HID_SEND_FEATURE_REPORT hid_send_feature_report;
    HID_GET_FEATURE_REPORT hid_get_feature_report;


public:
    hidControl()
    {}

    ~hidControl()
    {}

    unsigned char buf[REPORT_LENGTH+1];
    bool deviceFound;
    hid_device *handle;
    QString error_usb;

    // Инициализация HID
    int init(unsigned int VID, unsigned int PID)
    {


        QTextStream cout(stdout);
        // Пробуем подключить библиотеку
        QDir curdir;
        QLibrary lib(curdir.absolutePath() + "/hidapi");

        lib.load();
        setlocale(LC_ALL, "Russian");


        if (!lib.isLoaded()){
            cout << lib.errorString();
            error_usb = lib.errorString();
            //cout.flush();
            return -1;
        }

        hid_open = (HID_OPEN)lib.resolve("hid_open");
        hid_set_nonblocking = (HID_SET_NONBLOCKING)lib.resolve("hid_set_nonblocking");
        hid_send_feature_report = (HID_SEND_FEATURE_REPORT)lib.resolve("hid_send_feature_report");
        hid_get_feature_report = (HID_GET_FEATURE_REPORT)lib.resolve("hid_get_feature_report");

        deviceFound = false;

        handle = hid_open(VID, PID, NULL);

        if (handle)
        {
            deviceFound = true;
            hid_set_nonblocking(handle, 1);
        }
        else
            return -1;

        return 0;
    }

    int write()
    {
        buf[0] = 0x00;  // Идентификатор посылки всегда 0х00

        return hid_send_feature_report(handle, buf, REPORT_LENGTH);
    }

    int read()
    {
        buf[0] = 0x00;  // Идентификатор посылки всегда 0х00

        hid_send_feature_report(handle, buf, REPORT_LENGTH);

       // usleep(1000);

        return hid_get_feature_report(handle, buf, REPORT_LENGTH);
    }

};

#endif // HIDCONTROL_H
