/* Name: main.c
 * Project: hid-data, example how to use HID for data transfer
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-11
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 */

/*
 This example should run on most AVRs with only little changes. No special
 hardware resources except INT0 are used. You may have to change usbconfig.h for
 different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
 at least be connected to INT0 as well.
 */

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/eeprom.h>
#include <string.h>


#include "spi.h"

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"        /* This is also an example for using debug macros */
#include "requests.h"       /* The custom request numbers we use */


// ADC0 - 0b00000001
// ADC1 - 0b00000010
// ADC2 - 0b00000100
// ADC2 - 0b00001000
#define ADC0_EN    0
#define ADC1_EN    1
#define ADC2_EN    2
#define ADC3_EN    3
// Быстрый режим ождин опрос ADC за проход (Если скорость более 250)
// FAST_REC - 0b1000xxxx
#define FAST_REC    7
// БЫстрый режим с уменьшенной частотой деления на ADC (Если скорость более 350)
// FAST_DIV - 0b0100xxxx
#define FAST_DIV    6

uint8_t adc0 = 0;
uint8_t adc1 = 0;
uint8_t adc2 = 0;
uint8_t adc3 = 0;

uint8_t fast_rec = 0;
uint8_t fast_div = 0;

////////////////////////////////////////////////////////////////////////////////
///////////////   ADC


void adc_init() {

	DDRC &= ~((1 << PC0) |(1 << PC1) |(1 << PC2) |(1 << PC3));
	PORTC &= ~((1 << PC0) |(1 << PC1) |(1 << PC2) |(1 << PC3));

	ADCSRA = (1 << ADEN); // Вкл  ADC

	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // Делитель 1024
	ADMUX = 0 << ADLAR; // Правосторонний, что бы читать оба байта
	ADMUX |= (0 << REFS1) | (1 << REFS0); /// ОТ VCC с конденсатором
}


uint32_t get_ADC_data(uint8_t ADC_NUMBER) {

	ADMUX = (ADMUX & 0b11100000) | ADC_NUMBER;

	if (fast_div){
		ADCSRA |= /*(1 << ADPS2) |*/ (1 << ADPS0); // Делитель 1024
		ADCSRA &= ~(1 << ADPS1);
	}else{
		ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // Делитель 1024
	}


	uint32_t res = 0;

	if (!fast_rec){
	for (uint8_t i = 0; i < 3; i++) {
		ADCSRA |= 1 << ADSC;		// Start Conversion
		while ((ADCSRA & 0x10) == 0)
			; //ждем, пока АЦП закончит преобразование (ADIF = 0)
		res += ADCW ;
	}

	res /= 3;
	}else{
		ADCSRA |= 1 << ADSC;		// Start Conversion
		while ((ADCSRA & 0x10) == 0)
			; //ждем, пока АЦП закончит преобразование (ADIF = 0)
		res = ADCW ;
	}

	return res;

}
/////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
//////////////  DDS
///////////////////////////////////////////////////////////////////////////////

volatile double thisFreq =0;
volatile uint8_t reset = 0;

#define BUS_PORT        PORTB
#define BUS_DDR         DDRB
#define DATA            3
#define FQ_UD           2
#define W_CLK           5
#define RESET           1

void AD9850_init(void) {
	BUS_PORT &= ~(1 << RESET);
	BUS_PORT |= 1 << RESET;
	BUS_PORT &= ~(1 << RESET);
	BUS_PORT &= ~(1 << DATA);
	BUS_PORT |= 1 << W_CLK;
	BUS_PORT &= ~(1 << W_CLK);
	BUS_PORT |= 1 << FQ_UD;
	BUS_PORT &= ~(1 << FQ_UD);
	BUS_PORT |= 1 << W_CLK;
	BUS_PORT &= ~(1 << W_CLK);
	BUS_PORT |= 1 << W_CLK;
	BUS_PORT &= ~(1 << W_CLK);
	BUS_PORT |= 1 << W_CLK;
	BUS_PORT &= ~(1 << W_CLK);
	BUS_PORT |= 1 << FQ_UD;
	BUS_PORT &= ~(1 << FQ_UD);
}

void AD9850_Powerdown(void) {
	unsigned char i;
	unsigned char cmd = 0x04;

	BUS_PORT |= 1 << FQ_UD;
	for (i = 0; i < 8; i++) {
		if (cmd & 1)
			BUS_PORT |= 1 << DATA;
		else
			BUS_PORT &= ~(1 << DATA);
		BUS_PORT |= 1 << W_CLK;
		if (i == 0)
			BUS_PORT &= ~(1 << FQ_UD);
		cmd >>= 1;
		BUS_PORT &= ~(1 << W_CLK);
	}
	BUS_PORT |= 1 << FQ_UD;
	BUS_PORT &= ~(1 << FQ_UD);
}

void AD9850_Set(double freq) {
	unsigned char i;
	unsigned long dataword;

	dataword = 4294967296 * freq / 125000000;

	cli();
	for (i = 0; i < 40; i++) {
		if (dataword & 1)
			BUS_PORT |= 1 << DATA;
		else
			BUS_PORT &= ~(1 << DATA);
		BUS_PORT |= 1 << W_CLK;
		dataword >>= 1;
		BUS_PORT &= ~(1 << W_CLK);
	}
	sei();
	BUS_PORT |= 1 << FQ_UD;
	BUS_PORT &= ~(1 << FQ_UD);
}

void AD9850_Set_Test(double freq) {
	unsigned char i;
	unsigned long dataword;
	cli();
	dataword = freq;
	for (i = 0; i < 40; i++) {
		if (dataword & 1)
			BUS_PORT |= 1 << DATA;
		else
			BUS_PORT &= ~(1 << DATA);
		BUS_PORT |= 1 << W_CLK;
		dataword >>= 1;
		BUS_PORT &= ~(1 << W_CLK);
	}
	BUS_PORT |= 1 << FQ_UD;
	BUS_PORT &= ~(1 << FQ_UD);
	sei();
}

void cpu_init(void) {
	BUS_DDR |= (1 << DATA) | (1 << FQ_UD) | (1 << W_CLK) | (1 << RESET);
	BUS_PORT &= ~((1 << DATA) | (1 << FQ_UD) | (1 << W_CLK) | (1 << RESET));
}

////////////////////////////////////////////////////////////////////////////////




/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */



struct dataexchange_t       // Описание структуры для передачи данных
{
	uint32_t data1; // freq OR ADC 1-2
	uint32_t data2; //adust OR ADC2-3
}typedef dataexchange;


volatile  dataexchange pdata = {0};

/* Эти переменные хранят статус текущей передачи */
static uchar    currentAddress;
static uchar    bytesRemaining;


const PROGMEM char usbHidReportDescriptor[22] = { // USB report descriptor         // Дескриптор описывает структуру пакета данных для обмена
    0x06, 0x00, 0xff,                       // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                             // USAGE (Vendor Usage 1)
    0xa1, 0x01,                             // COLLECTION (Application)
    0x15, 0x00,                             //    LOGICAL_MINIMUM (0)        // min. значение для данных
    0x26, 0xff, 0x00,                       //    LOGICAL_MAXIMUM (255)      // max. значение для данных, 255 тут не случайно, а чтобы уложиться в 1 байт
    0x75, 0x08,                             //    REPORT_SIZE (8)            // информация передается порциями, это размер одного "репорта" 8 бит
    0x95, sizeof(dataexchange),    //    REPORT_COUNT               // количество порций (у нашем примере = 3, описанная выше структура передастся за три репорта)
    0x09, 0x00,                             //    USAGE (Undefined)
    0xb2, 0x02, 0x01,                       //    FEATURE (Data,Var,Abs,Buf)
    0xc0                                    // END_COLLECTION
};
/* Здесь мы описали только один report, из-за чего не нужно использовать report-ID (он должен быть первым байтом).
 * С его помощью передадим 3 байта данных (размер одного REPORT_SIZE = 8 бит = 1 байт, их количество REPORT_COUNT = 3).
 */



/* usbFunctionRead() вызывается когда хост запрашивает порцию данных от устройства
 * Для дополнительной информации см. документацию в usbdrv.h
 */
uchar   usbFunctionRead(uchar *data, uchar len)
{
    if(len > bytesRemaining)
        len = bytesRemaining;

    uchar *buffer = (uchar*)&pdata;



    if(!currentAddress)        // Ни один кусок данных еще не прочитан.
    {                          // Заполним структуру для передачи
    	//pdata.freq = thisFreq;
    	//uint32_t ADC_RES=0;

    	pdata.data1 = 0;
    	if (adc0) pdata.data1 = get_ADC_data(0b0000) << 16;
    	if (adc1) pdata.data1 += (get_ADC_data(0b0001));
    	if (adc2) pdata.data2= get_ADC_data(0b0010) << 16;
    	if (adc3) pdata.data2+= get_ADC_data(0b0011);


    }

    uchar j;
    for(j=0; j<len; j++)
        data[j] = buffer[j+currentAddress];

    currentAddress += len;
    bytesRemaining -= len;
    return len;
}


/* usbFunctionWrite() вызывается когда хост отправляет порцию данных к устройству
 * Для дополнительной информации см. документацию в usbdrv.h
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
    if(bytesRemaining == 0)
        return 1;               /* конец передачи */

    if(len > bytesRemaining)
        len = bytesRemaining;

    uchar *buffer = (uchar*)&pdata;

    uchar j;
    for(j=0; j<len; j++)
        buffer[j+currentAddress] = data[j];

    currentAddress += len;
    bytesRemaining -= len;


    if(bytesRemaining == 0)     // Все данные получены
    {                           // Выставим значения на PORTB
    	if (pdata.data1 > 0){
    		thisFreq =  pdata.data1;
    		AD9850_Set(thisFreq);

    		if (pdata.data2 > 0){
    			/// Читаем настройки
    			adc0 = pdata.data2 & (1 << ADC0_EN);
    			adc1 = pdata.data2 & (1 << ADC1_EN);
    			adc2 = pdata.data2 & (1 << ADC2_EN);
    			adc3 = pdata.data2 & (1 << ADC3_EN);

    			fast_rec = pdata.data2 & (1 << FAST_REC);
    			fast_div = pdata.data2 & (1 << FAST_DIV);


    		}

    	}
    }

    return bytesRemaining == 0; /* 0 означает, что есть еще данные */
}

/* ------------------------------------------------------------------------- */

/* ------------------------------------------------------------------------- */


usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID устройство */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            // у нас только одна разновидность репорта, можем игнорировать report-ID
            bytesRemaining = sizeof(dataexchange);
            currentAddress = 0;
            return USB_NO_MSG;  // используем usbFunctionRead() для отправки данных хосту
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            // у нас только одна разновидность репорта, можем игнорировать report-ID
            bytesRemaining = sizeof(dataexchange);
            currentAddress = 0;
            return USB_NO_MSG;  // используем usbFunctionWrite() для получения данных от хоста
        }
    }else{
        /* остальные запросы мы просто игнорируем */
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */






int main(void) {

	cpu_init();

	AD9850_init();

		thisFreq= 1000000;

		AD9850_Set(thisFreq);
		adc_init();

////////////////////// USB ////////////////


		uchar   i;

		    wdt_enable(WDTO_1S);
		    /* Even if you don't use the watchdog, turn it off here. On newer devices,
		     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
		     */
		    /* RESET status: all port bits are inputs without pull-up.
		     * That's the way we need D+ and D-. Therefore we don't need any
		     * additional hardware initialization.
		     */
		    //odDebugInit();
		    //DBG1(0x00, 0, 0);       /* debug output: main starts */
		    usbInit();
		    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
		    i = 0;
		    while(--i){             /* fake USB disconnect for > 250 ms */
		        wdt_reset();
		        _delay_ms(1);
		    }
		    usbDeviceConnect();

		    sei();
		    //DBG1(0x01, 0, 0);       /* debug output: main loop starts */
		    for(;;){                /* main event loop */
		#if 0   /* this is a bit too aggressive for a debug output */
		       // DBG2(0x02, 0, 0);   /* debug output: main loop iterates */
		#endif
		        wdt_reset();
		        usbPoll();
		    }






////////////////////////////////////////////


	return 0;
}

/* ------------------------------------------------------------------------- */
